/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation;

import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * Implementations of this class will run a given {@link ObservationVisitor} on an {@link ObservationSet}. Each instance
 * of this class is bound to a given {@code ObservationSet}. Typically it can be created with code such as:
 * 
 * <pre>
 * ObservationSet observationSet = ...
 * ObservationSetTraverser traverser = observationSet.as(ObservationSetTraverser);
 * traverser.accept(new ObservationVisitor() { ... });
 * </pre>
 * 
 * @navassoc - "call" - ObservationVisitor
 * @navassoc - "traverse" - ObservationSet
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface ObservationSetTraverser
  {
    public static final Class<ObservationSetTraverser> ObservationSetTraverser = ObservationSetTraverser.class;
    
    /*******************************************************************************************************************
     *
     * Runs the given visitor to the {@link ObservationSet} bound to this runner.
     * 
     * @param  visitor  the visitor to run
     * @return          the visitor (for a fluent interface use)
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T extends ObservationVisitor> T accept (@Nonnull T visitor);
  }
